<?php

/**
 * @file robotagger_api.class.php.
 * The main interface for the RoboTagger web service.
 */

class Robotagger {

  private $defaults = array();
  private $response = NULL;
  public $data = NULL;

  public static function gethost() {
    return variable_get('robotagger_api_server', 'http://ws.robotagger.com:8080/WebService/request');
  }
  private function get_defaults() {
    return array(
      'protocol' => 'http',
      'host' => self::gethost(),
      'contentType' => 'TEXT/HTML',
      'outputFormat' => 'XML/RDF',
    );
  }

  function __construct($options = array()) {
    $this->defaults = array_merge($this->get_defaults(), $options);
  }

  public function analyze($content, $params, $lang) {
    if (empty($content)) {
      return;
    }
    $this->__analyze($content, $params, $lang);
    $this->__format();
    return $this;
  }

  private function __analyze($content, $params, $lang) {

    // prepare annotypes
    ob_start();
    foreach ($params as $annotype) include 'robotagger_api.annotype.xml.php';
    $annotypes = ob_get_contents();
    ob_end_clean();

    // prepare xml data
    $data = array(
      'api_key' => variable_get('robotagger_api_server_apikey', ''),
      'lang' => $lang,
      'annotypes' => $annotypes,
      'content' => $content,
    );
    // generate xml
    ob_start();
    include 'robotagger_api.xml.php';
    $xml = ob_get_contents();
    ob_end_clean();
    try {
      // query
      $headers  = array('Content-Type' => 'application/x-www-form-urlencoded');
      $data_enc = http_build_query(array('rtXMLRequest' => $xml), '', '&');
      $response = drupal_http_request($this->defaults['host'], array('headers' => $headers, 'method' => 'POST', 'data' => $data_enc, 'max_redirects' => 3));
      // check for errors
      if (isset($response->error)) {
        self::log_robotagger_api_error('robotagger_api', 'RoboTagger API', $response->code, $response->error);
        return;
      }
    } catch (Exception $err) {
      return;
    }
    $this->response = $response;
    return $this;
  }

  private function __format(){
    try {
      if (empty($this->response->data)) {
        return;
      }
      $result = array();
      $error = qp($this->response->data)->find('error')->text();
      if (!empty($error)) {
        self::log_robotagger_api_error('robotagger_api', 'RoboTagger API', $this->response->code, $error);
        return;
      }
      $xml = qp($this->response->data);
      $result['topic'] = qp($this->response->data)->find('topic')->text();
      $children = $xml->children();
      $DOMElements = $children->get();
      foreach ($DOMElements as $DOMElement) {
        $DOMNodeLists = $DOMElement->getElementsByTagName('annotation');
        foreach ($DOMNodeLists as $DOMNodeList) {
          $anno_type = $DOMNodeList->getAttribute('annoType');
          $result[$anno_type][] = array(
            'value' => $DOMNodeList->getAttribute('stringVal'),
            'occurences' => $DOMNodeList->getAttribute('occurrences'),
            'threshold' => $DOMNodeList->getAttribute('threshold'),
            'subtype' => $DOMNodeList->getAttribute('subtype'),
          );
        }
      }
      $this->data = $result;
      return $this;
    }
    catch (Exception $e) {
      return $this;
    }
  }

  private static function log_robotagger_api_error($type, $uiname, $code, $msg) {
    $msg = t('@uiname processing error: (@code - @error)', array('@uiname' => $uiname, '@code' => $code, '@error' => $msg));
    drupal_set_message($msg, 'error', FALSE);
    watchdog($type, 'RoboTagger processing error: (@code - @error)', array('@code' => $code, '@error' => $msg), WATCHDOG_ERROR);
  }

  public static function _rtc_get_annotypes() {
    $return = array();
    $annotypes = _robotagger_api_get_annotypes();
    $subtypes = array();
    foreach ($annotypes as $annotype) {
      $key = drupal_strtolower($annotype['name']);
      $return[$key] = $annotype['name'];
      if (!empty($annotype['subtypes'])) {
        foreach($annotype['subtypes'] as $subtype) {
          $subtypes[$key] = $subtype['values'];
        }
      }
    }
    return array($return, $subtypes);
  }

  public static function _robotagger_api_get_annotype_names($langcode) {
    $annotypes = array();
    $robotagger_annotypes = self::_robotagger_api_get_annotypes($langcode);
    foreach ($robotagger_annotypes as $robotagger_annotype) {
      $annotypes[] = array('name' => $robotagger_annotype['name'], 'description' => $robotagger_annotype['description']);
    }
    return $annotypes;
  }

  public static function _robotagger_api_validate_api_key($key, $host) {
    if (empty($host)) {
      $host = self::gethost();
    }
    $response = drupal_http_request($host.'?check_apikey=' . $key);
    return $response->data;
  }
  public static function _robotagger_api_get_annotypes($langcode) {
    if (($cache = cache_get('robotagger_vocs')) && !empty($cache->data)) {
      return $cache->data;
    }
    $response = drupal_http_request(self::gethost() . '?get_annotypes=1');
    if ($response->code != 200) {
      self::log_robotagger_api_error('robotagger_api', 'RoboTagger API', $response->code, $response->error);
      return array();
    }
    $vocs = array();
    $qp = new QueryPath($response->data, NULL, array('replace_entities' => TRUE));
    $children = $qp->children();
    $annotation_types = $children->toArray();
    foreach ($annotation_types as $i => $annotation_type) {
      $name = $annotation_type->getAttribute('name');
      $voc = array();
      $voc['name'] = $name;
      $descriptions = $annotation_type->getElementsByTagName('description');
      $voc['description'] = '';
      if (!empty($descriptions)) {
        $voc['description'] = $descriptions->item(0)->nodeValue;
      }
      $voc['subtypes'] = array();
      $annotation_subtypes = $annotation_type->getElementsByTagName('annotation_subtype');
      foreach ($annotation_subtypes as $annotation_subtype) {
        $subtype = array();
        if (is_object($annotation_subtype)) {
          $subtypename = $annotation_subtype->getAttribute('name');
          $subtype['name'] = $subtypename;
          $subtype_values = $annotation_subtype->getElementsByTagName('subtype_value');
          $descriptions = $annotation_subtype->getElementsByTagName('description');
          $subtype['description'] = '';
          if (!empty($descriptions)) {
            $subtype['description'] = $descriptions->item(0)->nodeValue;
          }
          $subtype['values'] = array();
          if (is_object($subtype_values)) {
            foreach ($subtype_values as $subtype_value) {
              $value = $subtype_value->nodeValue;
              $subtype['values'][drupal_strtolower($value)] = $value;
            }
          }
        }
        $voc['subtypes'][] = $subtype;
      }//foreach ($annotation_subtypes as $annotation_subtype) {
      $vocs[] = $voc;
    }//foreach ($annotation_types as $i => $annotation_type) {
    if (!empty($vocs)) {
      $expire = time() + 60*60*24*7;
      cache_set('robotagger_vocs', $vocs, 'cache', $expire);
    }
    return $vocs;
  }
}
